<?php

namespace Drupal\scribit_pro;

use Drupal\Core\StringTranslation\TranslationManager;
use Drupal\scribit_pro\ApiService;

/**
 * Class HelperService.
 */
class HelperService {

  /**
   * Drupal\scribit_pro\ApiService definition.
   *
   * @var \Drupal\scribit_pro\ApiService
   */
  protected $api;

  /**
   * Translation manager.
   *
   * @var \Drupal\Core\StringTranslation\TranslationManager
   */
  protected $translationManager;

  /**
   * Constructs a new HelperService object.
   */
  public function __construct(ApiService $scribit_pro_api, TranslationManager $translation_manager) {
    $this->api = $scribit_pro_api;
    $this->translationManager = $translation_manager;
  }

  public function getLocalScribitProUser() {
    return $this->api->getLocalScribitProUser();
  }

  public function getLocalScribitProUserId() {
    return $this->api->getLocalScribitProUserId();
  }

  public function postYoutubeVideo($video_url, $voice_name, $voice_language) {
    if ($video_id = $this->extractYouTubeIdFromUrl($video_url)) {
      return $this->api->postProject($this->prepareYouTubeVideoUrl($video_id), $video_id, $voice_name, $voice_language);
    }
  }

  public function getScribitProUserLoginStatus($username_key = NULL, $password_key = NULL) {
    return $this->api->testUserLogin($username_key, $password_key);
  }

  public function prepareYouTubeVideoUrl($video_id) {
    return 'https://www.youtube.com/watch?v=' . $video_id;
  }

  /**
   * Get YouTube id from video url.
   *
   * @param string $url
   *   YouTube url the video id needs exctracted from.
   *
   * @return string|null
   *   YouTube id.
   */
  public function extractYouTubeIdFromUrl($url) {
    preg_match('/(?<=(?:v|i)=)[a-zA-Z0-9-]+(?=&)|(?<=(?:v|i)\/)[^&\n]+|(?<=embed\/)[^"&\n]+|(?<=(?:v|i)=)[^&\n]+|(?<=youtu.be\/)[^&\n]+/', $url, $id);

    return $id[0] ? $id[0] : NULL;
  }

  /**
   * Get available languages for Scribit Pro.
   *
   * @return array
   *   Array containing available Scribit Pro Languages.
   */
  public function getLanguages() {
    return [
      'en_uk' => $this->t('English (EN)'),
      'nl_nl' => $this->t('Dutch (NL)'),
      'nl_be' => $this->t('Flemish (BE)'),
      'sv_se' => $this->t('Swedish (SE)'),
    ];
  }

  /**
   * Get available voices for Scribit Pro.
   *
   * @return array
   *   Array containing available Scribit Pro Voices.
   */
  public function getVoices() {
    return [
      'Guus' => $this->t('Male'),
      'Ilse' => $this->t('Female'),
    ];
  }

  public function hasCredentials() {
    return $this->api->hasCredentials();
  }

  public function getProjectByYouTubeUrl($youtube_url) {
    if ($youtube_id = $this->extractYouTubeIdFromUrl($youtube_url)) {
      return $this->api->getProjectByYouTubeId($youtube_id);
    }

    return NULL;
  }

  /**
   * Translates a string to the current language or to a given language.
   *
   * @source \Drupal\Core\StringTranslation\TranslationManager::translate wrapper for translate function.
   */
  private function t($string, array $args = [], array $options = []) {
    return $this->translationManager->translate($string, $args, $options);
  }

}
